==> sanctify-1.0.0.tar.gz: sanctify-1.0.0/sanctify/validator.py <==
from typing import Any

from email_validator import EmailNotValidError, EmailUndeliverableError, ValidatedEmail, validate_email
from loguru import logger

from sanctify.constants import ComparisonOperations
from sanctify.exception import ValidationError
from sanctify.transformer import Transformer
from sanctify.utils import calculate_age, match_comparision_operation


class Validator:
    @staticmethod
    def validate_comparison_operations(value: Any, comparison_operations: dict[str: int | None]) -> Any | Exception:
        """
        Validate a value against a set of comparison operations.

        Args:
            value: The value to be validated.
            comparison_operations: A dictionary of comparison operations and their operands.

        Returns:
            The validated value if all comparison operations pass.

        Raises:
            ValidationError: If any comparison operation fails.
        """
        result = True
        for operation, operand in comparison_operations.items():
            result = result and match_comparision_operation(value=value, operation=operation, operand=operand)

        if result is True:
            return value
        else:
            raise ValidationError(
                f"Comparison Failed for ({type(value) = }, {value = }) with  {comparison_operations = }"
            )

    @staticmethod
    def validate_age(
        value: str | int,
        date_order_tuple: tuple,
        comparison_operations: dict[str: int | None],
    ) -> str | int | Exception:
        """
        Validate the age based on comparison operations.

        Args:
            value: The value representing the date of birth.
            date_order_tuple: A tuple specifying the order of year, month, and day in the date.
            comparison_operations: A dictionary of comparison operations and their operands.

        Returns:
            The validated age value if all comparison operations pass.

        Raises:
            ValidationError: If any comparison operation fails.
        """
        parsed_datetime = Transformer.parse_date_from_string(
            value=value, date_order_tuple=date_order_tuple, return_datetime=True
        )
        age = calculate_age(date_of_birth=parsed_datetime.date())

        # Raises err on validation failure
        _ = Validator.validate_comparison_operations(value=age, comparison_operations=comparison_operations)

        return value

    @staticmethod
    def validate_email(
        value: str, check_deliverability: bool = False
    ) -> str | EmailNotValidError | EmailUndeliverableError | Exception:
        """
        Validate an email address.

        Args:
            value: The email address to be validated.
            check_deliverability: Boolean flag indicating whether to check email deliverability.

        Returns:
            The normalized and validated email address.

        Raises:
            ValidationError: If the email is not valid or undeliverable.
            Exception: If an unexpected exception occurs.
        """
        try:
            validated_email: ValidatedEmail = validate_email(email=value, check_deliverability=check_deliverability)
            return validated_email.normalized

        except (EmailNotValidError, EmailUndeliverableError) as err:
            # The email is not valid or undeliverable, handle the exception
            logger.exception(f"{str(err) = }")

            raise ValidationError(f"Invalid Email ({type(value)}), {value}) | {str(err) = }")

    @staticmethod
    def validate_length_of_string(value: str, length: int):
        """
        Validate the length of a string.

        Args:
            value: The string to be validated.
            length: The expected length of the string.

        Returns:
            The validated string length.

        Raises:
            ValidationError: If the string length is not equal to the expected length.
        """
        return Validator.validate_comparison_operations(
            value=len(str(value)),
            comparison_operations={ComparisonOperations.EQUALS.value: length},
        )

    @staticmethod
    def validate_ssn(value: str):
        """
        Validate a Social Security Number (SSN).

        Args:
            value: The SSN to be validated.

        Returns:
            The validated SSN.

        Raises:
            ValidationError: If the SSN is not valid.
        """
        # Extract the ssn
        digits_only = Transformer.extract_digits_from_string(value=value)

        # Validate based on the number of digits
        match len(digits_only):
            case 12:
                return digits_only[-4:]

            case 4:
                return digits_only

            case _:
                raise ValidationError(f"SSN Validation Failed: ({type(value)}, {value})")

    @staticmethod
    def validate_zip_code(value: str):
        """
        Validate a zip code.

        Args:
            value: The zip code to be validated.

        Returns:
            The validated zip code.

        Raises:
            ValidationError: If the zip code does not have 5 digits.
        """
        # Extract the zip code
        digits_only = Transformer.extract_digits_from_string(value=value)

        # Validate based on the number of digits
        match len(digits_only):
            case 5:
                return digits_only

            case _:
                raise ValidationError(
                    f"Zip Code Validation Failed: ({type(value)}, {value}) | ZipCode({digits_only}) does not have 5 digits (Actual: {len(digits_only)})"
                )

    @staticmethod
    def validate_country_code(value: str, comparison_operations: dict):
        country_code = Transformer.extract_country_code(value=value)
        return Validator.validate_comparison_operations(value=country_code, comparison_operations=comparison_operations)

    @staticmethod
    def validate_phone_number(value: str, country_code_equals: int | None = None) -> int | ValidationError:
        phone_number = Transformer.extract_phone_number(value=value)
        country_code = Transformer.extract_country_code(value=value)

        if isinstance(country_code_equals, int) and country_code != country_code_equals:
            raise ValidationError(f"Country Code for {value} do not match {country_code_equals}")
        return phone_number

==> sanctify-1.0.2.tar.gz: sanctify-1.0.2/sanctify/validator.py <==
from typing import Any

from email_validator import EmailNotValidError, EmailUndeliverableError, ValidatedEmail, validate_email
from loguru import logger

from sanctify.constants import ComparisonOperations
from sanctify.exception import ValidationError
from sanctify.transformer import Transformer
from sanctify.utils import calculate_age, match_comparision_operation


class Validator:
    @staticmethod
    def validate_comparison_operations(value: Any, comparison_operations: dict[str: int | None]) -> Any | Exception:
        """
        Validate a value against a set of comparison operations.

        Args:
            value: The value to be validated.
            comparison_operations: A dictionary of comparison operations and their operands.

        Returns:
            The validated value if all comparison operations pass.

        Raises:
            ValidationError: If any comparison operation fails.
        """
        result = True
        for operation, operand in comparison_operations.items():
            result = result and match_comparision_operation(value=value, operation=operation, operand=operand)

        if result is True:
            return value
        else:
            raise ValidationError(f"Comparison Failed for {comparison_operations = }")

    @staticmethod
    def validate_age(
        value: str | int,
        date_order_tuple: tuple,
        comparison_operations: dict[str: int | None] = None,
    ) -> str | int | Exception:
        """
        Validate the age based on comparison operations.

        Args:
            value: The value representing the date of birth.
            date_order_tuple: A tuple specifying the order of year, month, and day in the date.
            comparison_operations: A dictionary of comparison operations and their operands.

        Returns:
            The validated age value if all comparison operations pass.

        Raises:
            ValidationError: If any comparison operation fails.
        """
        parsed_datetime = Transformer.parse_date_from_string(
            value=value, date_order_tuple=date_order_tuple, return_datetime=True
        )
        age = calculate_age(date_of_birth=parsed_datetime.date())

        if isinstance(comparison_operations, dict):
            # Raises err on validation failure
            _ = Validator.validate_comparison_operations(value=age, comparison_operations=comparison_operations)
        else:
            try:
                # Raises err on validation failure
                _ = Validator.validate_comparison_operations(
                    value=age,
                    comparison_operations={
                        ComparisonOperations.GREATER_THAN_EQUALS.value: 18,
                        ComparisonOperations.LESS_THAN_EQUALS.value: 100,
                    },
                )
            except ValidationError:
                raise ValidationError("Age should be >=18 and <=100")

        return value

    @staticmethod
    def validate_due_date(
        value: str | int,
        date_order_tuple: tuple,
        comparison_operations: dict[str: int | None] = None,
    ) -> str | int | Exception:
        """
        Validate the due date based on comparison operations.

        Args:
            value: The value representing the date of birth.
            date_order_tuple: A tuple specifying the order of year, month, and day in the date.
            comparison_operations: A dictionary of comparison operations and their operands.

        Returns:
            The validated age value if all comparison operations pass.

        Raises:
            ValidationError: If any comparison operation fails.
        """
        parsed_datetime = Transformer.parse_date_from_string(
            value=value, date_order_tuple=date_order_tuple, return_datetime=True
        )
        age = calculate_age(date_of_birth=parsed_datetime.date())

        if isinstance(comparison_operations, dict):
            # Raises err on validation failure
            _ = Validator.validate_comparison_operations(value=age, comparison_operations=comparison_operations)
        else:
            try:
                # Raises err on validation failure
                _ = Validator.validate_comparison_operations(
                    value=age,
                    comparison_operations={
                        ComparisonOperations.LESS_THAN_EQUALS.value: 10,
                    },
                )

            except ValidationError:
                raise ValidationError("Should be less than 10 years")

        return value

    @staticmethod
    def validate_email(
        value: str, check_deliverability: bool = False
    ) -> str | EmailNotValidError | EmailUndeliverableError | Exception:
        """
        Validate an email address.

        Args:
            value: The email address to be validated.
            check_deliverability: Boolean flag indicating whether to check email deliverability.

        Returns:
            The normalized and validated email address.

        Raises:
            ValidationError: If the email is not valid or undeliverable.
            Exception: If an unexpected exception occurs.
        """
        try:
            validated_email: ValidatedEmail = validate_email(email=value, check_deliverability=check_deliverability)
            return validated_email.normalized

        except (EmailNotValidError, EmailUndeliverableError) as err:
            # The email is not valid or undeliverable, handle the exception
            logger.exception(f"{str(err) = }")

            raise ValidationError(f"Invalid Email: {str(err)}")

    @staticmethod
    def validate_length_of_string(value: str, length: int):
        """
        Validate the length of a string.

        Args:
            value: The string to be validated.
            length: The expected length of the string.

        Returns:
            The validated string length.

        Raises:
            ValidationError: If the string length is not equal to the expected length.
        """
        return Validator.validate_comparison_operations(
            value=len(str(value)),
            comparison_operations={ComparisonOperations.EQUALS.value: length},
        )

    @staticmethod
    def validate_ssn(value: str):
        """
        Validate a Social Security Number (SSN).

        Args:
            value: The SSN to be validated.

        Returns:
            The validated SSN.

        Raises:
            ValidationError: If the SSN is not valid.
        """
        # Extract the ssn
        digits_only = Transformer.extract_digits_from_string(value=value)

        # Validate based on the number of digits
        match len(digits_only):
            case 12:
                return digits_only[-4:]

            case 4:
                return digits_only

            case _:
                raise ValidationError("Should be either 12 or 4 digits")

    @staticmethod
    def validate_zip_code(value: str):
        """
        Validate a zip code.

        Args:
            value: The zip code to be validated.

        Returns:
            The validated zip code.

        Raises:
            ValidationError: If the zip code does not have 5 digits.
        """
        # Extract the zip code
        digits_only = Transformer.extract_digits_from_string(value=value)

        # Validate based on the number of digits
        match len(digits_only):
            case 5:
                return digits_only

            case _:
                raise ValidationError(f"ZipCode({digits_only}) does not have 5 digits (Actual: {len(digits_only)})")

    @staticmethod
    def validate_country_code(value: str, comparison_operations: dict):
        country_code = Transformer.extract_country_code(value=value)
        return Validator.validate_comparison_operations(value=country_code, comparison_operations=comparison_operations)

    @staticmethod
    def validate_phone_number(value: str, country_code_equals: int | None = None) -> int | ValidationError:
        phone_number = Transformer.extract_phone_number(value=value)
        country_code = Transformer.extract_country_code(value=value)

        if isinstance(country_code_equals, int) and country_code != country_code_equals:
            raise ValidationError(f"Country Code for {value} do not match {country_code_equals}")
        return phone_number

    @staticmethod
    def validate_phone_number_with_optional_country_code_check(
        value: str, country_code_equals: int | None = None
    ) -> int | ValidationError:
        phone_number = Transformer.extract_phone_number(value=value)
        country_code = None
        try:
            country_code = Transformer.extract_country_code(value=value)
        except ValidationError:
            pass
        else:
            if (
                isinstance(country_code_equals, int)
                and isinstance(country_code, int)
                and country_code != country_code_equals
            ):
                raise ValidationError(f"Country Code for {value} do not match {country_code_equals}")
        return phone_number
